@@ -19,7 +19,7 @@ class scheme:
         o = util.defaults(kwargs, {'L':1000, 'num_lumps':101, 'lump':10, 'diffusion':1, 'min_burst':1e-14})
         self.solver = solver
         self.crn = solver.crn
-        nodes = self.dict_value(o.num_lumps, util.isint, 'num_nodes should either be a positive integer or a mapping (e.g. dictionary) indexed by every species and with positive integers')
+        nodes = self.dict_value(o.num_lumps, util.isint, '"num_lumps" should either be a positive integer or a mapping (e.g. dictionary) indexed by every species and with positive integers')
         self.num_cont_nodes = tuple([nodes[s] for s in self.crn.species])
         self.num_nodes = tuple([nodes[s] for s in self.crn.species] + self.crn.discrete_truncs)
         self.nm = np.prod(self.num_nodes)
@@ -228,89 +228,78 @@ class explicit(scheme):
         rctns_bursts_semi_t = copy.deepcopy(rctns)
         t = False # is there any time-dependence
         t_updates = []
+        def set_zero(lsts,i,j):
+            for lst in lsts:
+                lst[i].set_rate(j, 0)
         for i in range(len(rctns)):
             for j in self.crn.discrete_states_prod:
                 rate = rctns[i].get_rate(j)
                 if isinstance(rate, non_local):
-                    rctns[i].set_rate(j, 0)
-                    rctns_t[i].set_rate(j, 0)
-                    rctns_semi_t[i].set_rate(j, 0)
+                    set_zero([rctns,rctns_t,rctns_semi_t],i,j)
                     # if the rate or burst size is zero, then remove the reaction altogether
                     if rate.a == 0 or rate.b == 0:
-                        rctns_bursts[i].set_rate(j, 0)
-                        rctns_bursts_semi_t[i].set_rate(j, 0)
-                        rctns_bursts_t[i].set_rate(j, 0)
+                        set_zero([rctns_bursts,rctns_bursts_semi_t,rctns_bursts_t],i,j)
                     # either both bursting rates and sizes are time-independent
                     elif hasattr(rate.a, 't') and rate.a.t == False and hasattr(rate.b, 't') and rate.b.t == False:
-                        rctns_bursts_t[i].set_rate(j, 0)
-                        rctns_bursts_semi_t[i].set_rate(j, 0)
+                        set_zero([rctns_bursts_t,rctns_bursts_semi_t],i,j)
                     # or they're both time-semi-dependent
                     elif hasattr(rate.a, 't') and util.islistts(rate.a.t) and hasattr(rate.b, 't') and util.islistts(rate.b.t):
                         t_updates += rate.a.t + rate.b.t
-                        rctns_bursts[i].set_rate(j, 0)
-                        rctns_bursts_t[i].set_rate(j, 0)
+                        set_zero([rctns_bursts,rctns_bursts_t],i,j)
                     # otherwise the reaction is considered time-dependent
                     else:
                         t = True
-                        rctns_bursts[i].set_rate(j, 0)
-                        rctns_bursts_semi_t[i].set_rate(j, 0)
+                        set_zero([rctns_bursts,rctns_bursts_semi_t],i,j)
                 else:
-                    rctns_bursts[i].set_rate(j, 0)
-                    rctns_bursts_semi_t[i].set_rate(j, 0)
-                    rctns_bursts_t[i].set_rate(j, 0)
+                    set_zero([rctns_bursts,rctns_bursts_semi_t,rctns_bursts_t],i,j)
                     if rate == 0:
-                        rctns[i].set_rate(j, 0)
-                        rctns_t[i].set_rate(j, 0)
-                        rctns_semi_t[i].set_rate(j, 0)
+                        set_zero([rctns,rctns_t,rctns_semi_t],i,j)
                     elif hasattr(rate, 't') and rate.t == False:
-                        rctns_semi_t[i].set_rate(j, 0)
-                        rctns_t[i].set_rate(j, 0)
+                        set_zero([rctns_semi_t,rctns_t],i,j)
                     elif hasattr(rate, 't') and util.islistts(rate.t):
                         t_updates += rate.t
-                        rctns[i].set_rate(j, 0)
-                        rctns_t[i].set_rate(j, 0)
+                        set_zero([rctns,rctns_t],i,j)
                     else:
                         t = True
-                        rctns[i].set_rate(j, 0)
-                        rctns_semi_t[i].set_rate(j, 0)
+                        set_zero([rctns,rctns_semi_t],i,j)
         return t, t_updates, rctns, rctns_semi_t, rctns_t, rctns_bursts, rctns_bursts_semi_t, rctns_bursts_t
     
     def build_scheme_part(self, scheme, reactions, add_fn, t):
         for rctn in reactions:
             e = rctn.get_disp()
+            norm_e = np.linalg.norm(e)
             for m in self.crn.discrete_states_prod:
                 rate = rctn.get_rate(m)
                 if rate == 0:
                     continue
-                norm_e_dx = np.linalg.norm(e)*self.dx
                 for ij_ in self.ijprod:
                     ij = ij_ + m
                     i = self.ij2i(ij)
-                    add_fn(scheme, rate, ij, i, e, norm_e_dx, t)
+                    add_fn(scheme, rate, ij, i, e, norm_e, self.dx, t)
 
-    def add_reaction(self, scheme, rate, ij, i, e, norm_e_dx, t):
+    def add_reaction(self, scheme, rate, ij, i, e, norm_e, dx, t):
         ratex = rate(self.ij2xarg(ij), self.L, t)
         # phi_(j+e) if it's in the domain
         ije = ij + e
         if self.isvalidij(ije):
-            scheme[i,self.ij2i(ije)] += self.inv2L / norm_e_dx * rate(self.ij2xarg(ije), self.L, t) / norm_e_dx
-            scheme[i,i] += - (1 + self.inv2L / norm_e_dx) * ratex / norm_e_dx
+            scheme[i,self.ij2i(ije)] += self.inv2L / dx * rate(self.ij2xarg(ije), self.L, t) / dx
+            scheme[i,i] += - (1 + self.inv2L / dx) * ratex / dx
         # phi_j if it's in the domain (includes terms of j-e)
         ije = ij - e
         if self.isvalidij(ije):
-            scheme[i,self.ij2i(ije)] += (1 + self.inv2L / norm_e_dx) * rate(self.ij2xarg(ije), self.L, t) / norm_e_dx
-            scheme[i,i] += - self.inv2L / norm_e_dx * ratex / norm_e_dx
+            scheme[i,self.ij2i(ije)] += (1 + self.inv2L / dx) * rate(self.ij2xarg(ije), self.L, t) / dx
+            scheme[i,i] += - self.inv2L / dx * ratex / dx
 
-    def add_nonlocal(self, scheme, rate, ij, i, e, norm_e_dx, t):
-        rate.add(self, scheme, rate, ij, i, e, norm_e_dx, t)
+    def add_nonlocal(self, scheme, rate, ij, i, e, norm_e, dx, t):
+        rate.add(self, scheme, rate, ij, i, e, norm_e, dx, t)
         
-    def add_burst(self, scheme, rate, ij, i, e, norm_e_dx, t):
+    def add_burst(self, scheme, rate, ij, i, e, norm_e, dx, t):
         arg = self.ij2xarg(ij)
         ratea = rate.a(arg, self.L, t)
         b = rate.b(arg, self.L, t) / self.L # from the evaluation below, we need to assure that b>1
         
-        ratexk = ratea * b * (1-np.exp(-norm_e_dx / b))**2 / norm_e_dx
-        prod = np.exp(-norm_e_dx / b)
+        ratexk = ratea * b * (1-np.exp(-norm_e*dx / b))**2 / (norm_e*dx)
+        prod = np.exp(-norm_e*dx / b)
         # number of jumps that can be made staying within the domain
         domain = int(np.floor(min([(self.num_nodes[ii]-1 - ij[ii]) / e[ii] for ii in e.nonzero()[0]])))
         # no need to add entries to the sparsity matrix when the coefficients are too small
@@ -323,7 +312,7 @@ class explicit(scheme):
             scheme[self.ij2i(ije),i] += ratexk
             ratexk *= prod
 
-    def add_fragment(self, scheme, rate, ij, i, e, norm_e_dx, t):
+    def add_fragment(self, scheme, rate, ij, i, e, norm_e, dx, t):
         arg = self.ij2xarg(ij)
         a = rate.a(arg, self.L, t)
         C = rate.b(arg, self.L, t) # cumulative probability of finding a single daughter of certain size
@@ -335,7 +324,7 @@ class explicit(scheme):
                 scheme[i,i] += -a*p
                 scheme[self.ij2i(ij-k*e),i] += 2*a*p # there are two daughters
 
-    def add_fragment_delta(self, scheme, rate, ij, i, e, norm_e_dx, t):
+    def add_fragment_delta(self, scheme, rate, ij, i, e, norm_e, dx, t):
         arg = self.ij2xarg(ij)
         a = rate.a(arg, self.L, t)
         c = rate.b(arg, self.L, t) # fraction of smallest size
@@ -347,7 +336,7 @@ class explicit(scheme):
                 scheme[i,i] += -a*p
                 scheme[self.ij2i(ij-k*e),i] += 2*a*p # there are two daughters
             
-    def add_discrete(self, scheme, rate, ij, i, e, norm_e_dx, t):
+    def add_discrete(self, scheme, rate, ij, i, e, norm_e, dx, t):
         ratex = rate(self.ij2xarg(ij), self.L, t)
         # for the discrete reactions, we have to impose both the + and the -, but we have already checked validity
         scheme[i,i] += -ratex
@@ -511,36 +500,36 @@ class explicit_LUD(explicit_first_order):
     def __init__(self, solver, **kwargs):
         super().__init__(solver, **kwargs)
 
-    def add_reaction(self, scheme, rate, ij, i, e, norm_e_dx, t):
+    def add_reaction(self, scheme, rate, ij, i, e, norm_e, dx, t):
         ratex = rate(self.ij2xarg(ij), self.L, t)
         # phi_(j+e) if it's in the domain
         if self.isvalidij(ij + 2*e):
-            scheme[i,self.ij2i(ij + 2*e)] += (.5) * rate(self.ij2xarg(ij + 2*e), self.L, t) / norm_e_dx
-            scheme[i,self.ij2i(ij + e)] += (-1.5 + self.inv2L / norm_e_dx) * rate(self.ij2xarg(ij + e), self.L, t) / norm_e_dx
-            scheme[i,i] += - (self.inv2L / norm_e_dx) * ratex / norm_e_dx
+            scheme[i,self.ij2i(ij + 2*e)] += (.5) * rate(self.ij2xarg(ij + 2*e), self.L, t) / dx
+            scheme[i,self.ij2i(ij + e)] += (-1.5 + self.inv2L / dx) * rate(self.ij2xarg(ij + e), self.L, t) / dx
+            scheme[i,i] += - (self.inv2L / dx) * ratex / dx
         # phi_j if it's in the domain (includes terms of j-e)
         if self.isvalidij(ij - e) and self.isvalidij(ij + e):
-            scheme[i,self.ij2i(ij + e)] += (-.5) * rate(self.ij2xarg(ij + e), self.L, t) / norm_e_dx
-            scheme[i,i] += (1.5 - self.inv2L / norm_e_dx) * ratex / norm_e_dx
-            scheme[i,self.ij2i(ij - e)] += (self.inv2L / norm_e_dx) * rate(self.ij2xarg(ij - e), self.L, t) / norm_e_dx
+            scheme[i,self.ij2i(ij + e)] += (-.5) * rate(self.ij2xarg(ij + e), self.L, t) / dx
+            scheme[i,i] += (1.5 - self.inv2L / dx) * ratex / dx
+            scheme[i,self.ij2i(ij - e)] += (self.inv2L / dx) * rate(self.ij2xarg(ij - e), self.L, t) / dx
                 
 class explicit_third_order(explicit_first_order):
     
     def __init__(self, solver, **kwargs):
         super().__init__(solver, **kwargs)
 
-    def add_reaction(self, scheme, rate, ij, i, e, norm_e_dx, t):
+    def add_reaction(self, scheme, rate, ij, i, e, norm_e, dx, t):
         ratex = rate(self.ij2xarg(ij), self.L, t)
         # phi_(j+e) if it's in the domain
         if self.isvalidij(ij + 2*e):
-            scheme[i,self.ij2i(ij + 2*e)] += (1/6) * rate(self.ij2xarg(ij + 2*e), self.L, t) / norm_e_dx
-            scheme[i,self.ij2i(ij + e)] += (-5/6 + self.inv2L / norm_e_dx) * rate(self.ij2xarg(ij + e), self.L, t) / norm_e_dx
-            scheme[i,i] += - (2/6 + self.inv2L / norm_e_dx) * ratex / norm_e_dx            
+            scheme[i,self.ij2i(ij + 2*e)] += (1/6) * rate(self.ij2xarg(ij + 2*e), self.L, t) / dx
+            scheme[i,self.ij2i(ij + e)] += (-5/6 + self.inv2L / dx) * rate(self.ij2xarg(ij + e), self.L, t) / dx
+            scheme[i,i] += - (2/6 + self.inv2L / dx) * ratex / dx            
         # phi_j if it's in the domain (includes terms of j-2e)        
         if self.isvalidij(ij - e) and self.isvalidij(ij + e):
-            scheme[i,self.ij2i(ij + e)] += (-1/6) * rate(self.ij2xarg(ij + e), self.L, t) / norm_e_dx
-            scheme[i,self.ij2i(ij - e)] += (2/6 + self.inv2L / norm_e_dx) * rate(self.ij2xarg(ij - e), self.L, t) / norm_e_dx
-            scheme[i,i] += (5/6 - self.inv2L / norm_e_dx) * ratex / norm_e_dx
+            scheme[i,self.ij2i(ij + e)] += (-1/6) * rate(self.ij2xarg(ij + e), self.L, t) / dx
+            scheme[i,self.ij2i(ij - e)] += (2/6 + self.inv2L / dx) * rate(self.ij2xarg(ij - e), self.L, t) / dx
+            scheme[i,i] += (5/6 - self.inv2L / dx) * ratex / dx
 
 class explicit_KT(explicit):
     
@@ -569,7 +558,7 @@ class explicit_KT(explicit):
         h = self.base_scheme.diagonal()
         sum_ris = np.sum(ris, axis=0)
 
-        self.max_dt = 1/(np.max(np.abs(h + sum_ris/(self.L*self.dx**2))) + 4*I*max_norm_ri/self.dx) / self.stepper.CFL()
+        self.max_dt = 1/(np.max(np.abs(h + sum_ris*self.diffusion/(self.L*self.dx**2))) + 4*I*max_norm_ri/self.dx) / self.stepper.CFL()
         
     def build_iterable(self):
         self.static_base_scheme, self.static_flux_bases = self.build_frame(self.reactions, self.discrete_reactions, self.bursts, 0)
@@ -631,31 +620,31 @@ class explicit_KT(explicit):
         return scheme, flux_bases
     
     # an alternative way to evaluate the (u_x) terms at the boundary
-    def add_d_p_trunc(self, scheme, rate, ij, i, e, norm_e_dx, t):
+    def add_d_p_trunc(self, scheme, rate, ij, i, e, norm_e, dx, t):
         if self.isvalidij(ij + e):
             scheme[i,self.ij2i(ij + e)] += 1
             scheme[i,i] += -1
             
-    def add_d_p(self, scheme, rate, ij, i, e, norm_e_dx, t):
+    def add_d_p(self, scheme, rate, ij, i, e, norm_e, dx, t):
         if self.isvalidij(ij + e):
             scheme[i,self.ij2i(ij + e)] += 1
         scheme[i,i] += -1
             
-    def add_shift_m(self, scheme, rate, ij, i, e, norm_e_dx, t):
+    def add_shift_m(self, scheme, rate, ij, i, e, norm_e, dx, t):
         if self.isvalidij(ij - e):
             scheme[i,self.ij2i(ij - e)] += 1
             
-    def add_rates(self, scheme, rate, ij, i, e, norm_e_dx, t):
+    def add_rates(self, scheme, rate, ij, i, e, norm_e, dx, t):
         if self.isvalidij(ij + e):
             x1 = self.ij2xarg(ij)
             x2 = self.ij2xarg(ij + e)
             x = {k:(x1[k]+x2[k])/2 for k in x1.keys()}
             scheme[i,i] += rate(x, self.L, t)
         
-    def add_diff_flux(self, scheme, rate, ij, i, e, norm_e_dx, t):
+    def add_diff_flux(self, scheme, rate, ij, i, e, norm_e, dx, t):
         if self.isvalidij(ij + e):
-            scheme[i,self.ij2i(ij + e)] += self.inv2L * rate(self.ij2xarg(ij + e), self.L, t) / norm_e_dx
-            scheme[i,i] += - self.inv2L * rate(self.ij2xarg(ij), self.L, t) / norm_e_dx
+            scheme[i,self.ij2i(ij + e)] += self.inv2L * rate(self.ij2xarg(ij + e), self.L, t) / dx
+            scheme[i,i] += - self.inv2L * rate(self.ij2xarg(ij), self.L, t) / dx
     
     def diagnostics(self, t=0, p=None):
         if p is None:
